unit uDummyPort;

interface

uses
  Windows, Classes, Registry;

Type
  TBaudRate = (br110, br300, br600, br1200, br2400, br4800, br9600, br14400,
    br19200, br38400, br56000, br57600, br115200, br128000, br256000);

  TCustomDummyComPort = class(TComponent)
  private
    FPort: string;
    FConnected: Boolean;
    FModemHandle: Cardinal;
    FBaudRate: TBaudRate;
    FOutBuffer: PChar;
    procedure SetPort(const Value: string);
    function GetConnected: boolean;
    procedure SetConnected(const Value: boolean);
    function CreateHandle: boolean;
    procedure DestroyHandle;
    procedure SetBaudRate(const Value: TBaudRate);
  protected

  public
    Constructor Create(AOwner: TComponent); override;
    Destructor Destroy; override;
    function Open: boolean;
    procedure Close;
    Function SetDTR(OnOff: Boolean): boolean;
    function SetRTS(OnOff: Boolean): boolean;
    function Write(const P: PChar; const Size: integer): boolean;
    function WriteStr(const S: String): boolean;
    property Connected: boolean read GetConnected write SetConnected;
    Property Port: string read FPort write SetPort;
    property BaudRate: TBaudRate read FBaudRate write SetBaudRate;
  end;

  TDummyCom = class(TCustomDummyComPort)
    Property Port;
    property BaudRate;
  end;

function EnumComPorts(Ports: TStrings): boolean;

const
  OutBufLen = 1024;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples',[TDummyCom]);
end;

function EnumComPorts(Ports: TStrings): boolean;
var
  Reg: TRegistry;
  Str, TmpStr: TStringList;
  i: integer;
begin
  Result:= False;
  Ports.Clear;
  Str:= TStringList.Create;
  TmpStr:= TStringList.Create;
  Reg:= TRegistry.Create;
  Reg.RootKey:= HKEY_LOCAL_MACHINE;
  if Reg.OpenKeyReadOnly('\HARDWARE\DEVICEMAP\SERIALCOMM') then
  begin
    Reg.GetValueNames(Str);
    for i:= 0 to Str.Count - 1 do TmpStr.Add(Reg.ReadString(Str.Strings[i]));
    //TmpStr.Sort;
    Ports.Assign(TmpStr);
    Result:= True;
  end;
  Reg.Free;
  Str.Free;
  TmpStr.Free;
end;

{ TCustomDummyComPort }

procedure TCustomDummyComPort.Close;
begin
  DestroyHandle;
  FConnected:= False;
end;

constructor TCustomDummyComPort.Create(AOwner: TComponent);
begin
  inherited;
  FModemHandle:= INVALID_HANDLE_VALUE;
  FConnected:= False;
  FPort:= 'COM1';
  BaudRate:= br9600;
  GetMem(FOutBuffer, OutBufLen);
end;

function TCustomDummyComPort.CreateHandle: boolean;
begin
  FModemHandle := CreateFile(
    PChar('\\.\' + FPort),
    GENERIC_READ or GENERIC_WRITE,
    0,
    nil,
    OPEN_EXISTING,
    FILE_FLAG_OVERLAPPED,
    0);

  Result:= FModemHandle <> INVALID_HANDLE_VALUE;
end;

destructor TCustomDummyComPort.Destroy;
begin
  Close;
  FreeMem(FOutBuffer);
  inherited;
end;

procedure TCustomDummyComPort.DestroyHandle;
begin
  if FModemHandle <> INVALID_HANDLE_VALUE then
  begin
    CloseHandle(FModemHandle);
    FModemHandle:= INVALID_HANDLE_VALUE;
  end;
end;

function TCustomDummyComPort.GetConnected: boolean;
begin
  Result:= FConnected;
end;

function TCustomDummyComPort.Open: boolean;
const
  dcb_Binary = $00000001;
  CBaudRate: array[TBaudRate] of Integer =
    (CBR_110, CBR_300, CBR_600, CBR_1200, CBR_2400, CBR_4800, CBR_9600,
     CBR_14400, CBR_19200, CBR_38400, CBR_56000, CBR_57600, CBR_115200,
     CBR_128000, CBR_256000);
var
  DCB: TDCB;
begin
  Result:= False;
  if FConnected then
  begin
    Result:= True;
    exit;
  end;
  if not CreateHandle then
    exit;

  //Apply DCB
  FillChar(DCB, SizeOf(TDCB), 0);
  DCB.DCBlength := SizeOf(TDCB);
  DCB.XonLim := 256;
  DCB.XoffLim := DCB.XonLim;
  DCB.EvtChar := #0;
  DCB.XonChar := #17;
  DCB.XoffChar := #19;
  DCB.Parity := NOPARITY;
  DCB.StopBits := ONESTOPBIT;
  DCB.BaudRate := CBaudRate[BaudRate];


  DCB.Flags := dcb_Binary; //dcb_binary
  DCB.ByteSize := 8;
  if SetCommState(FModemHandle, DCB) then
  begin
    FConnected:= True;
    Result:= True;
  end;
end;

procedure TCustomDummyComPort.SetBaudRate(const Value: TBaudRate);
begin
  FBaudRate := Value;
end;

procedure TCustomDummyComPort.SetConnected(const Value: boolean);
begin
  if Value then
    Open
  else
    Close;

end;

function TCustomDummyComPort.SetDTR(OnOff: Boolean): boolean;
var
  Act: DWORD;
begin
  Result:= False;
  if not FConnected then exit;

  if OnOff then
    Act := Windows.SETDTR
  else
    Act := Windows.CLRDTR;

  Result:= EscapeCommFunction(FModemHandle, Act);
end;

procedure TCustomDummyComPort.SetPort(const Value: string);
begin
  FPort := Value;
end;

function TCustomDummyComPort.SetRTS(OnOff: Boolean): boolean;
var
  Act: DWORD;
begin
  Result:= False;
  if not FConnected then exit;

  if OnOff then
    Act := Windows.SETRTS
  else
    Act := Windows.CLRRTS;

  Result:= EscapeCommFunction(FModemHandle, Act);
end;

function TCustomDummyComPort.Write(const P: PChar; const Size: integer): boolean;
var
  WriteOL: TOverlapped;
  RealWrite, Signaled, BytesTrans: Cardinal;
begin
  Result:= False;
  if (P = nil) or (not FConnected) then exit;

  FillChar(WriteOL, SizeOf(WriteOL), 0);
  WriteOL.hEvent:= CreateEvent(nil, True, True, nil);
  try
    WriteFile(FModemHandle, P^, Size, RealWrite, @WriteOL);
    Signaled:= WaitForSingleObject(WriteOL.hEvent, 3000);
    Result:=
        (Signaled = WAIT_OBJECT_0) and
        (GetOverlappedResult(FModemHandle, WriteOL, BytesTrans, False));
  finally
    CloseHandle(WriteOL.hEvent);
  end;
end;

function TCustomDummyComPort.WriteStr(const S: String): boolean;
begin
  Result:= Write(PChar(S), Length(S));
end;

end.
